<#
.SYNOPSIS

Powershell equivalent of Unix "file" command

.DESCRIPTION

The ps-file is the Powershell equivalent of Unix  "file"command.  It is used in forensics to identify the file by its signature rather than the extension

See:
 PowerShell for Malware Incident Response    Amazon Kindle eBook by MARK OSBORNE

.PARAMETER file

The file name of a file to be processed. 

.EXAMPLE

ps-file -file helloworld.exe

#>

param(   $file     )
#
function Get-Real-Header
{
       param(
             $path
       )
       # Get content of file (up to 20 bytes) for signature analysis
       ([Byte[]] $fileheader = Get-Content -Path $path -TotalCount 20 -Encoding Byte) |
       ForEach-Object {
             if( $_ -lt 16)
             {
                   $HeaderAsHexString += "0{0:X}" -f $_ 
             }
             else
             {
                   $HeaderAsHexString += "{0:X}" -f $_
             }
       }
return($HeaderAsHexString)
}

function Usage
{
write-host "Ps-file  "
write-host "Usage:"
write-host "              ps-file -file file-name                   "

}
 # validate file objects   
If    ( ! $file  )  {
write-host "File cannot be null"
Usage
exit 
}

If    ( ! (test-path $file )  ) {
write-host "File not found"
Usage
exit 
}
$ActualFileHeader = Get-Real-Header($file)

$mysigs  = @(
(504B030414000600, DOCX, MS Office 2007 documents ),
(BEBAFECA, DBA, Palm Desktop Calendar Archive ),
(00014244, DBA, Palm Desktop To Do Archive ),
(00014454, TDA, Palm Desktop Calendar Archive ),
(00000100, ico, Computer icon encoded in ICO file format ),
(........667479703367, 3gp, 3g2, 3rd Generation Partnership Project 3GPP and 3GPP2 multimedia files ),
(1F9D, z, tar.z, compressed file (often tar zip) using Lempel-Ziv-Welch algorithm ),
(1FA0, z, tar.z, Compressed file (often tar zip) using LZH algorithm ),
(4241434B4D494B454449534B, bac,"File or tape containing a backup done with AmiBack onan Amiga. It typically is paired with an index file (idx) with 
the table of contents. ),
(425A68, bz2,Compressed file using Bzip2 algorithm" ),
(474946383761, gif, Image file encoded in the Graphics Interchange Format (GIF) ),
(474946383961, gif, Image file encoded in the Graphics Interchange Format (GIF)" ),
(49492A00, tif, tiff, Tagged Image File Format ),
(4D4D002A, tif, tiff, Tagged Image File Format ),
(49492A00100000004352, cr2, Canon RAW Format Version 2 Canon's RAW format is based on the TIFF file form ),
(802A5FD7, cin, Kodak Cineon image ),
(53445058 , dpx, SMPTE DPX image ),
(58504453, dpx, SMPTE DPX image ),
(762F3101, exr, OpenEXR image ),
(425047FB, bpg, Better Portable Graphics forma ),
(FFD8FFDB, jpg, jpeg, JPEG raw or in the JFIF or Exif file format ),
(FFD8FFE0....4A4649460001, jpg, jpeg, JPEG raw or in the JFIF or Exif file format ),
(FFD8FFE1....457869660000, jpg, jpeg, JPEG raw or in the JFIF or Exif file format ),
(464F524D........494C424D, ilbm, lbm, ibm, iff, IFF Interleaved Bitmap Image ),
(464F524D........38535658, 8svx, 8sv, svx, snd, iff, IFF 8-Bit Sampled Voice ),
(464F524D........4143424D, acbm, iff, Amiga Contiguous Bitmap ),
(464F524D........414E424D, anbm, iff, IFF Animated Bitmap ),
(464F524D........414E494D, anim, iff, IFF CEL Animation ),
(464F524D........46415858, faxx, fax, iff, IFF Facsimile Image ),
(464F524D........46545854, ftxt, txt, iff, IFF Formatted Text ),
(464F524D........534D5553, smus, smu, mus iff", "IFF Simple Musical Score ),
(464F524D........434D5553, cmus, mus, iff, IFF Musical Score ),
(464F524D........5955564E, yuvn, yuv, iff, IFF YUV Image ),
(464F524D........46414E54, iff, Amiga Fantavision Movie ),
(464F524D........41494646, aiff, aif, aifc, snd, iff, Audio Interchange File Format ),
("494E4458", "idx", "Index file to a file or tape containing a backup done with AmiBack on anAmiga." ),
("4D5A", exe, DOS MZ executable file format and its descendants including NE andPE ),
(504B0304, zip, jar, odt, ods, odp, docx, xlsx, pptx, apk, zip file format and formats based on it, such as JAR, ODF, OOXML ),
( 504B0506 , zip, jar, odt, ods, odp, docx, xlsx, pptx, apk, (empty archive)zip file format and formats based on it, such as JAR, ODF, OOXML ),
( 504B0708 , zip, jar, odt, ods, odp, docx, xlsx, pptx, apk, (spanned archive) zip file format and formats based on it, such as JAR, ODF, OOXML ),
(526172211A0700, rar, RAR archive version 1.50 onward ),
(526172211A070100, rar, RAR archive version 5.0 onward ),
(7F454C46,ELF ,  Executable and Linkable Format ),
(89504E470D0A1A0A, png, Image encoded in the Portable Network Graphics format   .PNG ),
(CAFEBABE, class, Java class file, Mach-O Fat Binary ),
(EFBBBF, utf-8 , UTF-8 encoded Unicode byte order mark, commonly seen in text files. ),
(FEEDFACE,v , Mach-O binary (32-bit) ),
(FEEDFACF, b, Mach-O binary (64-bit) ),
(CEFAEDFE,b , Mach-O binary (reverse byte ordering scheme, 32-bit)  ),
(CFFAEDFE, b, Mach-O binary (reverse byte ordering scheme, 64-bit)  ),
(FFFE, txt, Byte-order mark for text file encoded in little-endian ),
(FFFE0000, txt, Byte-order mark for text file encoded in little-endian  ),
(25215053, ps, PostScript document ),
(25504446, pdf, PDF document ),
(3026B2758E66CF11A6D900AA0062CE6C, asf, wma, wmv, Advanced Systems Format ),
(2453444930303031, bbbb , System Deployment Image, a disk image format used by Microsoft ),
(4F676753, ogg, oga, ogv, Ogg, an open source media container format ),
(38425053, psd, Photoshop Document file Adobe Photoshop's native file format ),
(52494646........57415645, wav, Waveform Audio File Format ),
(52494646........41564920, avi, Audio Video Interleave video format ),
(FFFB, mp3, MPEG-1 Layer 3 file without an ID3 tag or with an ID3v1 tag (which's appended at the end of the file) ),
(494433, mp3, MP3 file with an ID3v2 container ),
(424D, bmp, dib, BMP file, a bitmap format used mostly in the Windows world ),
(664C6143, "flac", Free Lossless Audio Codec ),
(4D546864, mid, midi, MIDI sound file ),
(D0CF11E0A1B11AE1, doc, xls, ppt, msg, Microsoft Office documents ),
(6465780A30333500, dex, Dalvik Executable ),
(4B444D, vmdk, VMDK files  ),
(43723234, crx, Google Chrome extension[18] or packaged app ),
(41474433, fh8, FreeHand 8 document  ),
(05070000424F424F0507000000000000000000000001, cwk, AppleWorks 5 document ),
(0607E100424F424F0607E10000000000000000000001, cwk, AppleWorks 6 document ),
("8B455202000000", toast, Roxio Toast disc image file, also some .dmg-files begin with same bytes ),
(455202000000, toast, Roxio Toast disc image file, also some .dmg-files begin with same bytes ),
(7801730D626260, dmg, Apple Disk Image file ),
(78617221, xar, eXtensible ARchive format ),
(504D4F43434D4F43, dat, Windows Files And Settings Transfer Repository[24] See also USMT 3.0 (Win XP)[25] and USMT 4.0 (Win 7) User Guides ),
(4E45531A, nes, Nintendo Entertainment System ROM file ),
(746F7833, tox, Open source portable voxel file ),
(4D4C5649, MLV, Magic Lantern Video file ),
(44434D0150413330, bdc, Windows Update Binary Delta Compression ),
(377ABCAF271C, 7z, 7-Zip File Format ),
(1F8B, gz, GZIP ),
(04224D18, lz4, LZ4 Streaming Format Remark: LZ4 compressed data (file) format does not offer any magic bytes.  ),
(4D534346, cab, "Microsoft Cabinet file ),
(464C4946, flif, Free Lossless Image Format )
)

Write-host "ps-file - Windows file command - identify a file by its signature"
Write-host "Author: Mark Osborne"
Write-host "--------------------------------------------------------"
write-host "NB: .txt files dont have a file magic number/signature "
write-host "-----------------------------------------------------------"

$ext = $null
$desc  =$null
foreach ($y in $mysigs) 
   { 
# for each get extension
   $ValidFileHeader = $y[0]
    # check the first n bytes of file header contain valid signature
   $ValidFileHeader = "^" +  $ValidFileHeader 
    # if a known file header
    if  (   $ActualFileHeader -match  $ValidFileHeader )
         {
          write-host "Valid Signature found " 
            $ext = $y[1]
           $desc  = $y[2]
           $desc
        }   
 #end loop
} 

if  ( ! $desc )
{
          write-host "NO Valid Signature found " 
}
