<#
.SYNOPSIS

Powershell hashdiff command

.DESCRIPTION

The ps-hashdiff  is the Powershell equivalent of tripwire or Afick. It compares  sha1 check sums of the sysroot against a db generated on  a clean computer

Author: Mark Osborne

See:
 PowerShell for Malware Incident Response    Amazon Kindle eBook by MARK OSBORNE

.PARAMETER file

The file name of a file to be processed. 

.PARAMETER dir

The dir name to be processed. 

.EXAMPLE

 ps-hashdiff  -file helloworld.exe       

 # compare c:\windows\helloworld.exe to sha hash in db

.EXAMPLE

 ps-hashdiff   

#compare c:\windows (whole file tree)  to sha hash in db

#>

param(   $dir  ,   $file     )
#

function Usage
{
write-host "Usage:"
write-host "              ps-hashdiff -dir dir-name -file file-name                   "
write-host "-----------------------------------------------------------"
}
 # validate file objects   

If    ( $dir -eq $null )  {
$sysroot = $env:SystemRoot 
}
If    ( $file -eq $null )  {
$file  = "*"
$sysroot += "\" +  $file
}
else
{
$sysroot += "\" +  $file
If    ( ! (test-path $sysroot )  ) {
write-host "File not found"
Usage
exit 
}
}
$t = 0
write-host  "ps-hashdiff - file integrity checker                  "
write-host  "Author:  Mark Osborne                 "
write-host "-----------------------------------------------------------"

 $y = Get-Childitem $sysroot -Include *.dll,*.exe,*.sys  -recurse  -force  -erroraction silentlycontinue | Select-Object fullname 
$x = import-csv .\hashdb.csv
foreach ( $yy in $y ) {
[bool] $found = $False
 foreach ( $xx in $x ) {
     if (  $xx.fullname -eq  $yy.FullName ) {
    # write-host $xx $hash
     $hash = Get-FileHash  -algorithm sha1  $yy.FullName    
[bool] $found = $True
        if (   $hash.hash -eq $xx.hash  ) {
                 $t++
           } else {
                $t++
             write-host  "hash not match"  $t  $yy.FullName
             write-host $hash.hash   $xx.hash 
            }  #if
      break
     } # if
 }   #for
if ( !  $found ) { 
  $t++ 
write-host $yy.FullName    $t "not found"  }
}

