<#
.SYNOPSIS

Powershell equivalent of PEid

.DESCRIPTION

The ps-peinfo is a Powershell command to display details of an executable file

See:
 PowerShell for Malware Incident Response    Amazon Kindle eBook by MARK OSBORNE

.PARAMETER file

The file name of a file to be processed. 

.EXAMPLE

ps-peinfo -file helloworld.exe

#>
param(   $file  ,  $max=0  )
#
function Usage
{
write-host "Usage:"
write-host "              ps-peinfo -file file-name                   "
write-host "-----------------------------------------------------------"

}
 # validate file objects   
If    ( $file -eq $null )  {
write-host "File cannot be null"
Usage
exit 
}

If    ( ! (test-path $file )  ) {
write-host "File not found"
Usage
exit 
}
# Get content of each file for analysis
[Byte[]] $fileheader = Get-Content   -Path $file -Encoding Byte


Write-host "ps-peinfo - powershell PeID"
Write-host "Author: Mark Osborne"
Write-host "--------------------------------------------------------"

Write-host "Check File Signature (should be  MZ) "
$zz = ">"+  ([char]  $fileheader[0] )  + ( [char]  $fileheader[1]) +"<"
Write-host   "file signature: "  $zz
Write " "
If  (   !($fileheader[0]  -eq 0x4D -and   $fileheader[1]  -eq 0x5A ) )
          {
         Write "Invalid exe signature"
         exit
}
 
Write-host "Check for Standard DOS stub"
Write-host " "
$dostring = "  "
foreach ( $f in $fileheader[78..118] ){
$dostring +=  [char] $f
}
if  ( $dostring -like "*program cannot be run in DOS mode*" )
   {
    Write-host  "Standard DOS STUB found"
    $dostring
    }
Else
   {
    Write-host  "DOS STUB not found"
    }

Write-host " "
 
#get e_lfanew 
$pe_offset =  $fileheader[0x3c]
Write-host "Value in Pe offset  ( e_lfanew )"  $pe_offset

Write-host " "
[string] $xx =  ([char]  $fileheader[$pe_offset ])  + ( [char]  $fileheader[$pe_offset +1] ) 
 Write-host "PE Field contains:"  $xx
 
If ( $xx -eq "PE"  ) {
    Write-host "PE  (Portable Executable) format designed for all Microsoft Win32 systems"
    }
If ( $xx -eq "NE") {
   Write-host "NE indicates a 16-bit Windows NE file."
}
If ( $xx -eq "LE") {
   Write-host  "LE indicates a Windows 3.x virtual device driver (VxD)."
}
If ( $xx -eq "LX") {
   Write-host  "LX indicates  a file for OS/2 2.0."
}

Write-host "  "
 $machine = [int]  $fileheader[$pe_offset+4]
# 0x14d	Intel i860
#0x14c	Intel I386 (same ID used for 486 and 586)
#0x162	MIPS R3000
#0x166	MIPS R4000
#0x183	DEC Alpha AXP
$machineStr += "{0:x}" -f $machine
if (  $machine -eq 0x4d )  { write-host "Machine Type field= 0x14d	Intel i860" }

if (  $machine -eq 0x4c )  { write-host "Machine Type field= 0x14c	Intel I386 (same ID used for 486 and 586)" }

if (  $machine -eq 0x62 )  { write-host "Machine Type field= 0x162	MIPS R3000" }

if (  $machine -eq 0x66)  { write-host "Machine Type field= 0x166	MIPS R4000" }

if (  $machine -eq 0x83)  { write-host "Machine Type field= 0x183	DEC Alpha AXP" }

# get number of sections
# WORD NumberOfSections
#The number of sections in the file
Write-host " "
$numberOsections = [int]  $fileheader[$pe_offset+6]
Write-host "Number of sections (i.e. text or rdata etc)"    $numberOsections
#get machine time
#get compiletime 
 $xx = "0x"
 $xx += "{0:x}" -f $fileheader[$pe_offset+8+3]
 $xx += "{0:x}" -f $fileheader[$pe_offset+8+2]
 $xx += "{0:x}" -f $fileheader[$pe_offset+8+1]
$xx += "{0:x}" -f $fileheader[$pe_offset+8+0]
$a= [Convert]::ToString($xx,10)
[datetime]$origin = '1970-01-01 00:00:00'
Write-host  " "
Write-host "Link  / Compile date: "     $origin.addseconds($a)
# DWORD PointerToSymbolTable
#The file offset of the COFF symbol table. This field is only used in OBJ files and PE files with COFF debug information. PE files support multiple #debug formats, so debuggers should refer to the IMAGE_DIRECTORY_ENTRY_DEBUG entry in the data directory (defined later).
$PointerToSymbolTable = $fileheader[$pe_offset+12]
#DWORD NumberOfSymbols
#The number of symbols in the COFF symbol table. See above.
$NumberOfSymbols = $fileheader[$pe_offset+16]
#WORD SizeOfOptionalHeader
#The size of an optional header that can follow this structure. In OBJs, the field is 0. In executables, it is the size of the IMAGE_OPTIONAL_HEADER #structure that follows this structure.
#WORD Characteristics
#Flags with information about the file. Some important fields:
#0x0001
#There are no relocations in this file
#0x0002
#File is an executable image (not a OBJ or LIB)
#0x2000
# File is a dynamic-link library, not a program
#https://msdn.microsoft.com/en-gb/library/ms809762.aspx
#