<#

.SYNOPSIS

Powershell listing of prefetch programs loaded between a time range


.DESCRIPTION
Windows caches programs for quick loading as part of the prefetch process.   This cmdlet is a 
Powershell listing of prefetch programs loaded between a time range
It is used in forensics to produce a timeline of loaded programs


See:
 PowerShell for Malware Incident Response    Amazon Kindle eBook by MARK OSBORNE


.PARAMETER   $sdate 

Startdate of range to select files 

This is an optional parameter.

.PARAMETER    $stime  

Start time of range to select files 

This is an optional parameter.

.PARAMETER    $edate  

End date of range to select files 

This is an optional parameter.

.PARAMETER    $etime 

End time of range to select files 

This is an optional parameter.

.EXAMPLE

ps-prefetch.ps1

Programs loaded today

.EXAMPLE

ps-prefetch.ps1  -sdate 22/2/18 -stime 09:10:00 -edate 22/2/18 -etime 20:10:00 

Programs loaded on 22/2/18 9am to 8pm

.NOTES
date format  dd/mm/ccyy 
time format  hh:mm:ss
#>
param(   $sdate ,  $stime  , $edate  , $etime )
#

# file time line
if (  $sdate -eq $null ) {
 $sdate = "yesterday"
 $stime = "00:00:00"
$sdatetime = (get-date).adddays(-1)
}
else
{
if (  $stime  -eq $null ) {
 $stime = "00:00:00"
}
$sdatetime = get-date ( $sdate +  " " + $stime ) 
}

# file time line
if (  $edate -eq $null ) {
 $edate = "today"
 $etime = "23:59:59"
 $edatetime =get-date
}
else
{
if (  $etime  -eq $null ) {
 $etime = "23:59:59"
}
 $edatetime =get-date ($edate  +  " " + $etime ) 

}

 $xx = $env:SystemRoot
 $dir = $xx  + "\Prefetch"
Write-host "Ps-prefetch - Windows prefetch file analyser (requires admin rights)"
Write-host "Author: Mark Osborne"
write-host "sdate:" $sdate  "stime:" $stime "edate:" $edate "etime:" $etime 
Write-host "--------------------------------------------------------"
 $timeline = Get-ChildItem   $dir   -filter "*.pf"  -force -erroraction silentlycontinue | Where-Object { $_.Lastwritetime -gt  $sdatetime  -and $_.Lastwritetime -lt $edatetime }  |sort -property lastwritetime
foreach ( $aa in $timeline )
{
  $y = $aa.name  -replace "-.*\.pf", " "
write-host ">" $aa.lastwritetime  " "  $y
}


#  | % {"us date: " +  $_.lastwritetime    +" "+    $_.fullname +" "+ $_.mode   }
#-------------------
