<#

.SYNOPSIS

Powershell cmdlet equivalent of "Mactime"  only for the registry 



.DESCRIPTION

The ps-regtimeline is a "Mactime"  cmdlet which display registry keys updated within a certain period. It is used in forensics to produce a timeline of changed reg keys



.PARAMETER $dir   

The root key of the hive to be processed 



.PARAMETER   $sdate 

Startdate of range to select reg keys

This is an optional parameter.

.PARAMETER    $stime  

Start time of range to select reg keys

This is an optional parameter.

.PARAMETER    $edate  

End date of range to select regkeys

This is an optional parameter.

.PARAMETER    $etime 

End time of range to select reg keys

This is an optional parameter.

.EXAMPLE

ps-regtimeline 

Changed regkeys today

.EXAMPLE

ps-pstimeline -sdate 22/2/18 -stime 09:10:00 -edate 22/2/18 -etime 20:10:00 

Changed regkeys on 22/2/18 9am to 8pm

.NOTES
date format  dd/mm/ccyy 
time format  hh:mm:ss
#>
param(   $dir  =  "HKLM:\"  ,  $sdate ,  $stime  , $edate  , $etime )
#


# file time line
if (  $sdate -eq $null ) {
$sdatetime = (get-date).adddays(-1)
}
else
{
if (  $stime  -eq $null ) {
 $stime = "00:00:00"
}
$sdatetime = get-date ( $sdate +  " " + $stime ) 
}

# file time line
if (  $edate -eq $null ) {
 $edatetime =get-date
}
else
{
if (  $etime  -eq $null ) {
 $etime = "23:59:59"
}
 $edatetime =get-date ($edate  +  " " + $etime ) 

}

Write-host "Ps-regtimeline.ps1 - Windows regtimeline analyser (requires admin rights)"
Write-host "Author: Mark Osborne"
Write-host "--------------------------------------------------------"
Write-host "List Registry keys under the hive " $dir 
Write-host  " Written between "    $sdatetime   " and " $edatetime

unblock-file .\Get-RegistryKeyLastWriteTime.ps1
Import-module  .\Get-RegistryKeyLastWriteTime.psm1
#  get-childitem "HKLM:\" -recurse | get-registrykeytimestamp | where-object { $_.lastwritetime -gt (get-date).adddays(-1) }

 $timeline =  get-childitem  $dir -recurse -erroraction silentlycontinue | get-registrykeytimestamp | Where-Object { $_.Lastwritetime -gt  $sdatetime  -and $_.Lastwritetime   -lt $edatetime }  |sort -property lastwritetime
foreach ( $tl in $timeline) {
  write-host $tl.lastwritetime  $tl.fullname
  }

# $timeline 
# | out-string -Width 160  
# | % {" " +  $_.lastwritetime + " " + $_.name     +" "+    $_.fullname   }
# $timeline | % {"us date: " +  $_.lastwritetime    +" "+    $_.fullname +" "+ $_.mode   }


 # $timeline = get-childitem  $dir -recurse -erroraction silentlycontinue | Where-Object { $_.Lastwritetime -gt  $sdatetime  -and $_.Lastwritetime #  -lt $edatetime }  |sort -property lastwritetime
# $timeline | % {"us date: " +  $_.lastwritetime    +" "+    $_.fullname +" "+ $_.mode   }
#-------------------
