<#

.SYNOPSIS

Powershell equivalent of Unix "strings" command



.DESCRIPTION

The ps-strings is the Powershell equivalent of Unix "strings" command.  It is used in forensics to hunt for clues in binary files 

See:
 PowerShell for Malware Incident Response    Amazon Kindle eBook by MARK OSBORNE

.PARAMETER path

The path and file name of a file to be processed. Any computers that cannot be reached will be logged to this file. 



.PARAMETER charset

The charset controls if unicode or ascii charset is used - ascii is the default.

This is an optional parameter.

.EXAMPLE

ps-strings -path helloworld.exe


.NOTES

A string is 2 or more consequetive chars

#>
param(   $path, $charset='ascii'     )

function Usage
{
write-host "Ps-strings  "
write-host "Usage:"
write-host "              ps-strings -path file-name                   "
write-host "-----------------------------------------------------------"
}
 # validate file objects   
If    ( $path -eq $null )  {
write-host "File cannot be null"
Usage
exit 
}

If    ( ! (test-path $path )  ) {
write-host "File not found"
Usage
exit 
}
$chars = @( "ascii", "unicode" )

If    ( ! ($charset -in $chars  )  ) {
write-host "charset not valid valid values "
$chars
Usage
exit 
}

Write-host "ps-strings - Windows strings"
Write-host "Author: Mark Osborne"
Write-host "--------------------------------------------------------"
 select-string -Path $path  -Encoding $charset   -Pattern "[0-9A-Za-z_ .\/\\]{2,100}" -Allmatches | foreach-object { $_.matches.value }

