﻿
<#

.SYNOPSIS

Powershell listing of running processes


.DESCRIPTION

ps-tasklist is a Powershell  cmdlet listing of  running processes
It is used in forensics to produce a listing of loaded programs which could be malware

See:
 PowerShell for Malware Incident Response   – Amazon Kindle eBook by MARK OSBORNE


.PARAMETER   $verbose 

List paths commandline and parent process for each process

This is an optional parameter.


.EXAMPLE

ps-tasklist

Programs runing

.EXAMPLE

ps-tasklist.ps1  -modules  -verbose

All modules loaded and command line options

#>
# 
param(   [switch]$modules ,  [switch]$verbose )
#

write-host $modules 
write-host  $verbose 

Write-host "ps-tasklist - Powershell listing of running processes"
Write-host "Author: Mark Osborne"
Write-host "--------------------------------------------------------"
 
$y =" "
 $aa = Get-Process
 foreach ( $a in $aa) {
    $y = " "
    if  ( $a.path -ne $null ) {
         if  ( test-path $a.path  ) {
              $x =Get-AuthenticodeSignature  $a.path
              $y = $x.status 
             if ( $x.status -eq "Valid" ) {
                  $y = "Signed"
                  } #if
         } #if
      } #if
   Write-host  $a.id $a.name $a.path  $y $a.starttime 
   write-host $a.description
   
   if ( $modules -eq $true) {
                        if ( $a.Modules.count -gt 0 ) {
	       write-host "----------------------------------------"
                           write-host  "  Size(K) ModuleName                                         FileName"
   	      $a.Modules
                        }
       }  
    if ( $verbose -eq $true) {
                   write-host "----------------------------------------"
                    $w = " " 
                   $w =  Get-WmiObject -Class Win32_Process | Where-Object { $_.processid -eq $a.id }
     
                   write-host "ParentprocessId "  $w.ParentprocessId
                    write-host "Commandline "  $w.commandline
                    write-host "Executablepath " $w.executablepath
    }
    write-host "----------------------------------------"
   write-host "----------------------------------------"
 }  #for