<#

.SYNOPSIS

Powershell equivalent of "Mactime"  command



.DESCRIPTION

The ps-timeline is the Powershell equivalent of "Mactime"  command.  It is used in forensics to produce a timeline of changed file

See:
 PowerShell for Malware Incident Response    Amazon Kindle eBook by MARK OSBORNE

.PARAMETER $dir   

The path of filesystem tree to be processed 



.PARAMETER   $sdate 

Startdate of range to select files 

This is an optional parameter.

.PARAMETER    $stime  

Start time of range to select files 

This is an optional parameter.

.PARAMETER    $edate  

End date of range to select files 

This is an optional parameter.

.PARAMETER    $etime 

End time of range to select files 

This is an optional parameter.

.EXAMPLE

ps-timeline -dir c:\  

Changed files today

.EXAMPLE

ps-timeline -sdate 22/2/18 -stime 09:10:00 -edate 22/2/18 -etime 20:10:00 

Changed files on 22/2/18 9am to 8pm

.NOTES
date format  dd/mm/ccyy 
time format  hh:mm:ss
#>
param(   $dir  =  "C:\users\"   ,  $sdate ,  $stime  , $edate  , $etime )
#

If ( -not ( test-path $dir ))
{
write-host "Invalid path specified"
write-host "ps-timeline -dir c:\users -sdate 1/1/1970 -stime 0:00:00 -edate 1/2/1970 -etime 23:00:00 "
exit
}

# file time line
if (  $sdate -eq $null ) {
$sdatetime = (get-date).adddays(-1)
}
else
{
if (  $stime  -eq $null ) {
 $stime = "00:00:00"
}
$sdatetime = get-date ( $sdate +  " " + $stime ) 
}

# file time line
if (  $edate -eq $null ) {
 $edatetime =get-date
}
else
{
if (  $etime  -eq $null ) {
 $etime = "23:59:59"
}
 $edatetime =get-date ($edate  +  " " + $etime ) 

}

Write-host "ps-timeline -  Powershell equivalent of  Mactime  command"
Write-host "Author: Mark Osborne"
Write-host "--------------------------------------------------------"
Write-host "List file in the " $dir 
Write-host  " Written between "    $sdatetime   " and " $edatetime

 $timeline = get-childitem  $dir -recurse  -force -include *  -erroraction silentlycontinue | Where-Object { $_.Lastwritetime -gt  $sdatetime  -and $_.Lastwritetime -lt $edatetime }  |sort -property lastwritetime
$timeline | % {"us date: " +  $_.lastwritetime    +" "+    $_.fullname +" "+ $_.mode   }
#-------------------
